package com.ruoyi.web.controller.IndexController;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CollectionUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.IncomeQuery;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Calendar;
import java.util.List;

/**
 * index首页统计
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/index")
public class IndexController extends BaseController
{

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private ISysUserService sysUserService;

    /**
     * 根据token获取运营区列表（后台）
     */
    @GetMapping("/getAreaList")
    public AjaxResult getAreaList()
    {
        List<EtOperatingArea> longs;
        SysUser sysUser = sysUserService.selectUserById(getUserId());
        if(getUserId() == 1){
            longs = etOperatingAreaService.selectAreaListByDeptId2(null,null);
        }else{
            if(ObjectUtil.isNotNull(sysUser.getAreaId())){
                longs = etOperatingAreaService.selectAreaListByDeptId2(null,sysUser.getAreaId());
            }else{
                if(ObjectUtil.isNotNull(sysUser.getDeptId())){
                    longs = etOperatingAreaService.selectAreaListByDeptId2(sysUser.getDeptId(),null);
                }else{
                    throw new ServiceException("该用户都没有绑定运营区和运营商（至少绑定一个）");
                }
            }
        }
        logger.info("根据token获取运营区列表：【{}】", JSON.toJSON(longs));
        return success(longs);
    }

    /**
     * 首页统计-运营商
     */
    @GetMapping(value = "/statistics")
    public AjaxResult statistics(Long areaId)
    {
        logger.info("【首页统计】请求参数: areaId={}", areaId);
        IndexVo indexVo = etOrderService.statistics(areaId,"1");
        return success(indexVo);
    }

    /**
     * 首页统计-运营商
     */
    @GetMapping(value = "/statisticsByDeptId")
    public AjaxResult statisticsByDeptId(Long deptId)
    {
        logger.info("【运营商统计】请求参数: deptId={}", deptId);
        if(deptId == null){
            throw new ServiceException("运营商id不能为空：deptId");
        }
        List<Long> areaIds = etOperatingAreaService.selectAreaListByDeptId(deptId);
        // 创建一个结果对象
        IndexVo result = new IndexVo();
        for (Long areaId : areaIds) {
            IndexVo areaIndexVo = etOrderService.statistics(areaId,"2");
            // 赋值
            assign(result, areaIndexVo);
        }
        return success(result);
    }

    /**
     * 获取营收统计（按月份）
     */
    @GetMapping(value = "/getIncomeList")
    public AjaxResult getIncomeList(Long deptId,String date)// 2024-11
    {
        logger.info("【获取营收统计（按月份）】请求参数: deptId={}", deptId);
        if(deptId == null){
            throw new ServiceException("运营商id不能为空：deptId");
        }
        if(StrUtil.isBlank(date)){
            throw new ServiceException("时间参数不能为空：date");
        }
        LocalDate[] firstAndLastDay =DateUtils.getFirstAndLastDayOfMonth(date);
        LocalDate startTime = firstAndLastDay[0];
        LocalDate endTime = firstAndLastDay[1];

        IncomeQuery incomeQuery = new IncomeQuery();
        incomeQuery.setDeptId(deptId);
        incomeQuery.setStartTime(startTime);
        incomeQuery.setEndTime(endTime);
        List<IncomeVo> incomeList = etOrderService.getIncomeList(incomeQuery);
        incomeList = CollectionUtils.fillVoids(incomeList, IncomeVo::getDay, (d) -> {
            IncomeVo vo = new IncomeVo();
            vo.setDay(d);
            vo.setIncome(BigDecimal.ZERO);
            vo.setOrderNum(0);
            return vo;
        }, startTime, endTime);
        return success(incomeList);
    }

    private String getDateStr(Calendar calendar,int day) {
        calendar.set(Calendar.DAY_OF_MONTH, day);
        return new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
    }

    private static void assign(IndexVo result, IndexVo areaIndexVo) {
        result.setBalance(result.getBalance().add(areaIndexVo.getBalance()));
        result.setWithdraw(result.getWithdraw().add(areaIndexVo.getWithdraw()));
        result.setSettlementAmount(result.getSettlementAmount().add(areaIndexVo.getSettlementAmount()));
        result.setTodayIncome(result.getTodayIncome().add(areaIndexVo.getTodayIncome()));
        result.setTodayOrderFee(result.getTodayOrderFee().add(areaIndexVo.getTodayOrderFee()));
        result.setTodayOrderCount(result.getTodayOrderCount() + areaIndexVo.getTodayOrderCount());
        result.setTotalIncome(result.getTotalIncome().add(areaIndexVo.getTotalIncome()));
        result.setTotalOrderCount(result.getTotalOrderCount() + areaIndexVo.getTotalOrderCount());
        result.setTotalOrderFee(result.getTotalOrderFee().add(areaIndexVo.getTotalOrderFee()));
        result.setUnpaidOrderCount(result.getUnpaidOrderCount() + areaIndexVo.getUnpaidOrderCount());
        result.setUnpaidOrderFee(result.getUnpaidOrderFee().add(areaIndexVo.getUnpaidOrderFee()));
        result.setReturnOrderCount(result.getReturnOrderCount() + areaIndexVo.getReturnOrderCount());
        result.setRidingOrder(result.getRidingOrder() + areaIndexVo.getRidingOrder());
        result.setTodayRefundFee(result.getTodayRefundFee().add(areaIndexVo.getTodayRefundFee()));
        result.setFaultOrderCount(result.getFaultOrderCount() + areaIndexVo.getFaultOrderCount());
        result.setVehicleVo(areaIndexVo.getVehicleVo());
        result.setIncomeVoList(areaIndexVo.getIncomeVoList());
    }


    /**
     * 首页统计-总管理
     */
    @GetMapping(value = "/admim/statistics")
    public AjaxResult admimStatistics()
    {
        logger.info("【首页统计】请求参数");
        IndexAdminVo indexAdminVo = etOrderService.admimStatistics();
        return success(indexAdminVo);
    }


    /**
     * 首页统计-排行榜
     * type：1-根据订单数排序，2-根据订单金额排序; 3-进行中的订单
     * timeLimit：1-今天，2-本周
     */
    @GetMapping(value = "/admim/leaderboard")
    public AjaxResult leaderboard(String type,String timeLimit)
    {
        logger.info("【首页统计】排行榜");
        List<LeaderboardVo> leaderboard = etOrderService.leaderboard(type, timeLimit);
        return success(leaderboard);
    }

}
